@@ -9,7 +9,7 @@
  **  This source code is part of the Valentina project, a pattern making
  **  program, whose allow create and modeling patterns of clothing.
  **  Copyright (C) 2015 Valentina project
- **  <https://bitbucket.org/dismine/valentina> All Rights Reserved.
+ **  <https://gitlab.com/smart-pattern/valentina> All Rights Reserved.
  **
  **  Valentina is free software: you can redistribute it and/or modify
  **  it under the terms of the GNU General Public License as published by
@@ -43,19 +43,20 @@
 #include "../vmisc/def.h"
 #include "vdomdocument.h"
 #include "vtoolrecord.h"
+#include "../vlayout/vtextmanager.h"
 
 class QDomElement;
 class VPiecePath;
 class VPieceNode;
 
-enum class Document : char { LiteParse, LitePPParse, FullParse };
-enum class LabelType : char {NewPatternPiece, NewLabel};
+enum class Document : qint8 { FullLiteParse, LiteParse, LitePPParse, FullParse };
+enum class LabelType : qint8 {NewPatternPiece, NewLabel};
 
 // Don't touch values!!!. Same values stored in xml.
-enum class CrossCirclesPoint : char {FirstPoint = 1, SecondPoint = 2};
-enum class VCrossCurvesPoint : char {HighestPoint = 1, LowestPoint = 2};
-enum class HCrossCurvesPoint : char {LeftmostPoint = 1, RightmostPoint = 2};
-enum class AxisType : char {VerticalAxis = 1, HorizontalAxis = 2};
+enum class CrossCirclesPoint : qint8 {FirstPoint = 1, SecondPoint = 2};
+enum class VCrossCurvesPoint : qint8 {HighestPoint = 1, LowestPoint = 2};
+enum class HCrossCurvesPoint : qint8 {LeftmostPoint = 1, RightmostPoint = 2};
+enum class AxisType : qint8 {VerticalAxis = 1, HorizontalAxis = 2};
 
 class VContainer;
 class VDataTool;
@@ -77,6 +78,13 @@ struct VFinalMeasurement
     QString description;
 };
 
+struct VGroupData
+{
+    QString name{};
+    bool visible{true};
+    QStringList tags{};
+};
+
 QT_WARNING_POP
 
 class VAbstractPattern : public VDomDocument
@@ -86,6 +94,7 @@ public:
     explicit VAbstractPattern(QObject *parent = nullptr);
     virtual ~VAbstractPattern();
 
+    bool           RequiresMeasurements() const;
     QStringList    ListMeasurements() const;
     QVector<VFormulaField> ListExpressions() const;
     QVector<VFormulaField> ListIncrementExpressions() const;
@@ -105,6 +114,8 @@ public:
     quint32        getCursor() const;
     void           setCursor(const quint32 &value);
 
+    virtual void   setXMLContent(const QString &fileName) override;
+
     virtual void   IncrementReferens(quint32 id) const=0;
     virtual void   DecrementReferens(quint32 id) const=0;
 
@@ -113,6 +124,8 @@ public:
 
     virtual void   UpdateToolData(const quint32 &id, VContainer *data)=0;
 
+    virtual void Clear();
+
     static VDataTool* getTool(quint32 id);
     static void       AddTool(quint32 id, VDataTool *tool);
     static void       RemoveTool(quint32 id);
@@ -134,12 +147,6 @@ public:
 
     QStringList    getPatternPieces() const;
 
-    QMap<GHeights, bool> GetGradationHeights() const;
-    void                 SetGradationHeights(const QMap<GHeights, bool> &options);
-
-    QMap<GSizes, bool>   GetGradationSizes() const;
-    void                 SetGradationSizes(const QMap<GSizes, bool> &options);
-
     QString        GetDescription() const;
     void           SetDescription(const QString &text);
 
@@ -158,6 +165,12 @@ public:
     QString        GetCustomerName() const;
     void           SetCustomerName(const QString& qsName);
 
+    QDate          GetCustomerBirthDate() const;
+    void           SetCustomerBirthDate(const QDate& date);
+
+    QString        GetCustomerEmail() const;
+    void           SetCustomerEmail(const QString& email);
+
     QString        GetLabelDateFormat() const;
     void           SetLabelDateFormat(const QString &format);
 
@@ -167,6 +180,9 @@ public:
     void                        SetPatternLabelTemplate(const QVector<VLabelTemplateLine> &lines);
     QVector<VLabelTemplateLine> GetPatternLabelTemplate() const;
 
+    bool SetWatermarkPath(const QString &path);
+    QString GetWatermarkPath() const;
+
     void               SetPatternMaterials(const QMap<int, QString> &materials);
     QMap<int, QString> GetPatternMaterials() const;
 
@@ -191,19 +207,32 @@ public:
 
     void           ParseGroups(const QDomElement &domElement);
     QDomElement    CreateGroups();
-    QDomElement    CreateGroup(quint32 id, const QString &name, const QMap<quint32, quint32> &groupData);
+    QDomElement    CreateGroup(quint32 id, const QString &name, const QStringList &tags,
+                               const QMap<quint32, quint32> &groupData, vidtype tool=null_id);
+    vidtype        GroupLinkedToTool(vidtype toolId) const;
+
     QString        GetGroupName(quint32 id);
     void           SetGroupName(quint32 id, const QString &name);
-    QMap<quint32, QPair<QString, bool> > GetGroups();
+
+    QStringList    GetGroupTags(vidtype id);
+    void           SetGroupTags(quint32 id, const QStringList &tags);
+
+    QStringList GetGroupCategories() const;
+
+    QMap<quint32, VGroupData> GetGroups();
     QMap<quint32, QString> GetGroupsContainingItem(quint32 toolId, quint32 objectId, bool containItem);
     QDomElement           AddItemToGroup(quint32 toolId, quint32 objectId, quint32 groupId);
     QDomElement           RemoveItemFromGroup(quint32 toolId, quint32 objectId, quint32 groupId);
     bool           GroupIsEmpty(quint32 id);
-    bool           GetGroupVisivility(quint32 id);
-    void           SetGroupVisivility(quint32 id, bool visible);
+    bool           GetGroupVisibility(quint32 id);
+
+    static QStringList FilterGroupTags(const QString &tags);
 
     QString PieceDrawName(quint32 id);
 
+    virtual VContainer GetCompleteData() const;
+    virtual VContainer GetCompletePPData(const QString &name) const;
+
     static const QString TagPattern;
     static const QString TagCalculation;
     static const QString TagModeling;
@@ -226,16 +255,16 @@ public:
     static const QString TagElArc;
     static const QString TagTools;
     static const QString TagOperation;
-    static const QString TagGradation;
-    static const QString TagHeights;
-    static const QString TagSizes;
     static const QString TagData;
     static const QString TagPatternInfo;
     static const QString TagPatternName;
     static const QString TagPatternNum;
     static const QString TagCompanyName;
     static const QString TagCustomerName;
+    static const QString TagCustomerBirthDate;
+    static const QString TagCustomerEmail;
     static const QString TagPatternLabel;
+    static const QString TagWatermark;
     static const QString TagPatternMaterials;
     static const QString TagFinalMeasurements;
     static const QString TagMaterial;
@@ -276,66 +305,11 @@ public:
     static const QString AttrRotation;
     static const QString AttrNumber;
     static const QString AttrCheckUniqueness;
+    static const QString AttrManualPassmarkLength;
+    static const QString AttrPassmarkLength;
+    static const QString AttrOpacity;
+    static const QString AttrTags;
 
-    static const QString AttrAll;
-
-    static const QString AttrH50;
-    static const QString AttrH56;
-    static const QString AttrH62;
-    static const QString AttrH68;
-    static const QString AttrH74;
-    static const QString AttrH80;
-    static const QString AttrH86;
-    static const QString AttrH92;
-    static const QString AttrH98;
-    static const QString AttrH104;
-    static const QString AttrH110;
-    static const QString AttrH116;
-    static const QString AttrH122;
-    static const QString AttrH128;
-    static const QString AttrH134;
-    static const QString AttrH140;
-    static const QString AttrH146;
-    static const QString AttrH152;
-    static const QString AttrH158;
-    static const QString AttrH164;
-    static const QString AttrH170;
-    static const QString AttrH176;
-    static const QString AttrH182;
-    static const QString AttrH188;
-    static const QString AttrH194;
-    static const QString AttrH200;
-
-    static const QString AttrS22;
-    static const QString AttrS24;
-    static const QString AttrS26;
-    static const QString AttrS28;
-    static const QString AttrS30;
-    static const QString AttrS32;
-    static const QString AttrS34;
-    static const QString AttrS36;
-    static const QString AttrS38;
-    static const QString AttrS40;
-    static const QString AttrS42;
-    static const QString AttrS44;
-    static const QString AttrS46;
-    static const QString AttrS48;
-    static const QString AttrS50;
-    static const QString AttrS52;
-    static const QString AttrS54;
-    static const QString AttrS56;
-    static const QString AttrS58;
-    static const QString AttrS60;
-    static const QString AttrS62;
-    static const QString AttrS64;
-    static const QString AttrS66;
-    static const QString AttrS68;
-    static const QString AttrS70;
-    static const QString AttrS72;
-
-    static const QString AttrCustom;
-    static const QString AttrDefHeight;
-    static const QString AttrDefSize;
     static const QString AttrExtension;
 
     static const QString AttrFormula;
@@ -392,7 +366,8 @@ signals:
     /**
      * @brief UpdateGroups emit if the groups have been updated
      */
-    void            UpdateGroups();
+    void           UpdateGroups();
+    void           UpdateToolTip();
 
 public slots:
     virtual void   LiteParseTree(const Document &parse)=0;
@@ -401,6 +376,7 @@ public slots:
     void           ClearScene();
     void           CheckInLayoutList();
     void           SelectedDetail(quint32 id);
+    void           UpdateVisiblityGroups();
 
 protected:
     /** @brief nameActivDraw name current pattern peace. */
