@@ -9,7 +9,7 @@
  **  This source code is part of the Valentina project, a pattern making
  **  program, whose allow create and modeling patterns of clothing.
  **  Copyright (C) 2013-2015 Valentina project
- **  <https://bitbucket.org/dismine/valentina> All Rights Reserved.
+ **  <https://gitlab.com/smart-pattern/valentina> All Rights Reserved.
  **
  **  Valentina is free software: you can redistribute it and/or modify
  **  it under the terms of the GNU General Public License as published by
@@ -55,13 +55,14 @@
 #include "../vdrawtool.h"
 
 //---------------------------------------------------------------------------------------------------------------------
-VAbstractSpline::VAbstractSpline(VAbstractPattern *doc, VContainer *data, quint32 id, QGraphicsItem *parent)
-    :VDrawTool(doc, data, id),
+VAbstractSpline::VAbstractSpline(VAbstractPattern *doc, VContainer *data, quint32 id, const QString &notes,
+                                 QGraphicsItem *parent)
+    :VDrawTool(doc, data, id, notes),
       QGraphicsPathItem(parent),
       controlPoints(),
       sceneType(SceneObject::Unknown),
       m_isHovered(false),
-      detailsMode(qApp->Settings()->IsShowCurveDetails()),
+      detailsMode(VAbstractApplication::VApp()->Settings()->IsShowCurveDetails()),
       m_acceptHoverEvents(true),
       m_parentRefresh(false)
 {
@@ -95,7 +96,8 @@ QPainterPath VAbstractSpline::shape() const
 //---------------------------------------------------------------------------------------------------------------------
 void VAbstractSpline::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-    const qreal width = ScaleWidth(m_isHovered ? qApp->Settings()->WidthMainLine() : qApp->Settings()->WidthHairLine(),
+    const qreal width = ScaleWidth(m_isHovered ? VAbstractApplication::VApp()->Settings()->WidthMainLine()
+                                               : VAbstractApplication::VApp()->Settings()->WidthHairLine(),
                                    SceneScale(scene()));
 
     const QSharedPointer<VAbstractCurve> curve = VAbstractTool::data.GeometricObject<VAbstractCurve>(m_id);
@@ -201,8 +203,8 @@ QString VAbstractSpline::MakeToolTip() const
                                     "<tr> <td><b>%1:</b> %2 %3</td> </tr>"
                                     "</table>")
             .arg(tr("Length"))
-            .arg(qApp->fromPixel(curve->GetLength()))
-            .arg(UnitsToStr(qApp->patternUnit(), true), tr("Label"), curve->name());
+            .arg(VAbstractValApplication::VApp()->fromPixel(curve->GetLength()))
+            .arg(UnitsToStr(VAbstractValApplication::VApp()->patternUnits(), true), tr("Label"), curve->ObjectName());
     return toolTip;
 }
 
@@ -322,12 +324,6 @@ void VAbstractSpline::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     QGraphicsPathItem::mouseReleaseEvent(event);
 }
 
-//---------------------------------------------------------------------------------------------------------------------
-void VAbstractSpline::ReadToolAttributes(const QDomElement &domElement)
-{
-    Q_UNUSED(domElement)
-}
-
 //---------------------------------------------------------------------------------------------------------------------
 void VAbstractSpline::SaveOptions(QDomElement &tag, QSharedPointer<VGObject> &obj)
 {
@@ -337,6 +333,7 @@ void VAbstractSpline::SaveOptions(QDomElement &tag, QSharedPointer<VGObject> &ob
     doc->SetAttribute(tag, AttrColor, curve->GetColor());
     doc->SetAttribute(tag, AttrPenStyle, curve->GetPenStyle());
     doc->SetAttribute(tag, AttrAScale, curve->GetApproximationScale());
+    doc->SetAttributeOrRemoveIf(tag, AttrAlias, curve->GetAliasSuffix(), curve->GetAliasSuffix().isEmpty());
 }
 
 //---------------------------------------------------------------------------------------------------------------------
@@ -368,7 +365,7 @@ VSpline VAbstractSpline::CorrectedSpline(const VSpline &spline, const SplinePoin
         QString newAngle1F = QString().setNum(newAngle1);
 
         qreal newLength1 = line.length();
-        QString newLength1F = QString().setNum(qApp->fromPixel(newLength1));
+        QString newLength1F = QString().setNum(VAbstractValApplication::VApp()->fromPixel(newLength1));
 
         if (not qmu::QmuTokenParser::IsSingle(spline.GetStartAngleFormula()))
         {
@@ -398,7 +395,7 @@ VSpline VAbstractSpline::CorrectedSpline(const VSpline &spline, const SplinePoin
         QString newAngle2F = QString().setNum(newAngle2);
 
         qreal newLength2 = line.length();
-        QString newLength2F = QString().setNum(qApp->fromPixel(newLength2));
+        QString newLength2F = QString().setNum(VAbstractValApplication::VApp()->fromPixel(newLength2));
 
         if (not qmu::QmuTokenParser::IsSingle(spline.GetEndAngleFormula()))
         {
@@ -487,6 +484,46 @@ QString VAbstractSpline::name() const
     return ObjectName<VAbstractCurve>(m_id);
 }
 
+//---------------------------------------------------------------------------------------------------------------------
+qreal VAbstractSpline::GetApproximationScale() const
+{
+    return VAbstractTool::data.GeometricObject<VAbstractCurve>(m_id)->GetApproximationScale();
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+quint32 VAbstractSpline::GetDuplicate() const
+{
+    return VAbstractTool::data.GeometricObject<VAbstractCurve>(m_id)->GetDuplicate();
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+QString VAbstractSpline::GetAliasSuffix() const
+{
+    return ObjectAliasSuffix<VAbstractCurve>(m_id);
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+void VAbstractSpline::SetAliasSuffix(QString alias)
+{
+    QSharedPointer<VAbstractCurve> curve = VAbstractTool::data.GeometricObject<VAbstractCurve>(m_id);
+
+    const QString oldAliasSuffix = curve->GetAliasSuffix();
+    alias = alias.simplified().replace(QChar(QChar::Space), QChar('_'));
+    curve->SetAliasSuffix(alias);
+
+    QRegularExpression rx(NameRegExp());
+
+    if (alias.isEmpty() || (rx.match(curve->GetAlias()).hasMatch() && VAbstractTool::data.IsUnique(curve->GetAlias())))
+    {
+        QSharedPointer<VGObject> obj = qSharedPointerCast<VGObject>(curve);
+        SaveOption(obj);
+    }
+    else
+    {
+        curve->SetAliasSuffix(oldAliasSuffix);
+    }
+}
+
 //---------------------------------------------------------------------------------------------------------------------
 void VAbstractSpline::GroupVisibility(quint32 object, bool visible)
 {
@@ -496,8 +533,9 @@ void VAbstractSpline::GroupVisibility(quint32 object, bool visible)
 
 // VToolAbstractArc
 //---------------------------------------------------------------------------------------------------------------------
-VToolAbstractArc::VToolAbstractArc(VAbstractPattern *doc, VContainer *data, quint32 id, QGraphicsItem *parent)
-    : VAbstractSpline(doc, data, id, parent)
+VToolAbstractArc::VToolAbstractArc(VAbstractPattern *doc, VContainer *data, quint32 id, const QString &notes,
+                                   QGraphicsItem *parent)
+    : VAbstractSpline(doc, data, id, notes, parent)
 {}
 
 //---------------------------------------------------------------------------------------------------------------------
