@@ -9,7 +9,7 @@
  **  This source code is part of the Valentina project, a pattern making
  **  program, whose allow create and modeling patterns of clothing.
  **  Copyright (C) 2015 Valentina project
- **  <https://bitbucket.org/dismine/valentina> All Rights Reserved.
+ **  <https://gitlab.com/smart-pattern/valentina> All Rights Reserved.
  **
  **  Valentina is free software: you can redistribute it and/or modify
  **  it under the terms of the GNU General Public License as published by
@@ -29,13 +29,36 @@
 #include "tst_varc.h"
 #include "../vgeometry/varc.h"
 #include "../vlayout/vabstractpiece.h"
-#include "../vmisc/logging.h"
 
 #include <QtTest>
 
+namespace
+{
+void PrepareTestCase(const QPointF &center, qreal startAngle, qreal endAngle)
+{
+    qreal radius = UnitConvertor(1, Unit::Cm, Unit::Px);
+    const qreal threshold = UnitConvertor(2000, Unit::Cm, Unit::Px);
+    while(radius <= threshold)
+    {
+        VArc arc(VPointF(center), radius, startAngle, endAngle);
+        const QVector<QPointF> points = arc.GetPoints();
+
+        const QString testStartAngle = QString("Test start angel. Arc radius %1, start angle %2, end angle %3")
+                                           .arg(radius).arg(startAngle).arg(endAngle);
+        QTest::newRow(qUtf8Printable(testStartAngle)) << center << startAngle << points << points.first() << true;
+
+        const QString testEndAngle = QString("Test end angel. Arc radius %1, start angle %2, end angle %3")
+                                         .arg(radius).arg(startAngle).arg(endAngle);
+        QTest::newRow(qUtf8Printable(testEndAngle)) << center << endAngle << points << points.last() << true;
+
+        radius += UnitConvertor(5, Unit::Cm, Unit::Px);
+    }
+}
+}
+
 //---------------------------------------------------------------------------------------------------------------------
 TST_VArc::TST_VArc(QObject *parent)
-    :QObject(parent)
+    :AbstractTest(parent)
 {}
 
 //---------------------------------------------------------------------------------------------------------------------
@@ -318,3 +341,118 @@ void TST_VArc::TestFlip()
     QCOMPARE(originArc.GetRadius(), res.GetRadius());
     QCOMPARE(originArc.AngleArc(), res.AngleArc());
 }
+
+//---------------------------------------------------------------------------------------------------------------------
+void TST_VArc::TestCutArc_data()
+{
+    QTest::addColumn<QPointF>("center");
+    QTest::addColumn<qreal>("radius");
+    QTest::addColumn<qreal>("startAngle");
+    QTest::addColumn<qreal>("length");
+    QTest::addColumn<qreal>("cutLength");
+    QTest::addColumn<QPointF>("cutPoint");
+
+    QPointF center (189.13625196850393, 344.1267401574803);
+    qreal radius = ToPixel(10, Unit::Cm);
+    qreal startAngle = 45.0;
+    qreal length = ToPixel(-10, Unit::Cm);
+    qreal cutLength = ToPixel(6, Unit::Cm);
+    QPointF cutPoint(539.3657292513009, 202.04366960088566);
+
+    // See file <root>/src/app/share/collection/bugs/Issue_#957.val
+    QTest::newRow("Arc -10 cm length, cut length 6 cm") << center << radius << startAngle << length << cutLength
+                                                        << cutPoint;
+
+    cutLength = ToPixel(-4, Unit::Cm);
+
+    // Opposite case
+    QTest::newRow("Arc -10 cm length, cut length -4 cm") << center << radius << startAngle << length << cutLength
+                                                         << cutPoint;
+
+    startAngle = 135;
+    length = ToPixel(10, Unit::Cm);
+    cutLength = ToPixel(-7, Unit::Cm);
+    cutPoint = QPointF(-145.1588983496871, 167.78888781060192);
+
+    // See file <root>/src/app/share/collection/bugs/Issue_#957.val
+    QTest::newRow("Arc 10 cm length, cut length -7 cm") << center << radius << startAngle << length << cutLength
+                                                        << cutPoint;
+
+    // Opposite case
+    cutLength = ToPixel(3, Unit::Cm);
+    QTest::newRow("Arc 10 cm length, cut length 3 cm") << center << radius << startAngle << length << cutLength
+                                                       << cutPoint;
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+void TST_VArc::TestCutArc()
+{
+    QFETCH(QPointF, center);
+    QFETCH(qreal, radius);
+    QFETCH(qreal, startAngle);
+    QFETCH(qreal, length);
+    QFETCH(qreal, cutLength);
+    QFETCH(QPointF, cutPoint);
+
+    VArc arc (length, VPointF(center), radius, startAngle);
+    arc.SetApproximationScale(0.5);
+
+    VArc arc1;
+    VArc arc2;
+    QPointF point = arc.CutArc(cutLength, arc1, arc2);
+
+    QCOMPARE(point, cutPoint);
+
+    QCOMPARE(arc1.IsFlipped(), arc.IsFlipped());
+    QCOMPARE(arc2.IsFlipped(), arc.IsFlipped());
+
+    QCOMPARE(arc1.GetApproximationScale(), arc.GetApproximationScale());
+    QCOMPARE(arc2.GetApproximationScale(), arc.GetApproximationScale());
+
+    QCOMPARE(arc1.GetStartAngle(), arc.GetStartAngle());
+    QCOMPARE(arc2.GetEndAngle(), arc.GetEndAngle());
+
+    QVERIFY(VFuzzyComparePossibleNulls(arc1.GetEndAngle(), arc2.GetStartAngle()));
+
+    QCOMPARE(arc1.GetLength() + arc2.GetLength(), arc.GetLength());
+
+    qreal segmentsLength = VAbstractCurve::PathLength(arc1.GetPoints()) + VAbstractCurve::PathLength(arc2.GetPoints());
+    QVERIFY(qAbs(segmentsLength - VAbstractCurve::PathLength(arc.GetPoints())) <= accuracyPointOnLine);
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+void TST_VArc::TestCurveIntersectAxis_data()
+{
+    QTest::addColumn<QPointF>("basePoint");
+    QTest::addColumn<qreal>("angle");
+    QTest::addColumn<QVector<QPointF>>("curvePoints");
+    QTest::addColumn<QPointF>("crosPoint");
+    QTest::addColumn<bool>("result");
+
+    QPointF basePoint(10, 10);
+
+    PrepareTestCase(basePoint, 0, 15);
+    PrepareTestCase(basePoint, 0, 25);
+    PrepareTestCase(basePoint, 0, 45);
+    PrepareTestCase(basePoint, 0, 90);
+    PrepareTestCase(basePoint, 0, 180);
+    PrepareTestCase(basePoint, 0, 270);
+    PrepareTestCase(basePoint, 180, 315);
+    PrepareTestCase(basePoint, 183, 312);
+}
+
+//---------------------------------------------------------------------------------------------------------------------
+void TST_VArc::TestCurveIntersectAxis()
+{
+    QFETCH(QPointF, basePoint);
+    QFETCH(qreal, angle);
+    QFETCH(QVector<QPointF>, curvePoints);
+    QFETCH(QPointF, crosPoint);
+    QFETCH(bool, result);
+
+    QPointF intersectionPoint;
+    const bool found = VAbstractCurve::CurveIntersectAxis(basePoint, angle, curvePoints, &intersectionPoint);
+    QCOMPARE(found, result);
+
+    Comparison(intersectionPoint, crosPoint);
+}
